from remotemanager.serialisation.yaml import serialyaml
from remotemanager.serialisation.json import serialjson

__all__ = ['serialyaml', 'serialjson']

try:
    from remotemanager.serialisation.dill import serialdill
    __all__.append('serialdill')
except ImportError:
    pass



import dill

from remotemanager.utils import ensure_filetype
from remotemanager.serialisation.serial import serial


class serialdill(serial):
    """
    subclass of serial, implementing dill methods
    """

    def dump(self, obj, file):
        file = ensure_filetype(file, self.extension)
        with open(file, 'wb+') as ofile:
            dill.dump(obj, ofile)

    def load(self, file):
        file = ensure_filetype(file, self.extension)
        with open(file, 'rb') as ofile:
            data = dill.load(ofile)
        return data

    @property
    def extension(self):
        return ".dill"

    @property
    def importstring(self):
        return "import dill"

    @property
    def callstring(self):
        return "dill"

    @property
    def bytes(self):
        return True



import yaml

from remotemanager.utils import ensure_filetype
from remotemanager.serialisation.serial import serial


class serialyaml(serial):
    """
    subclass of serial, implementing yaml methods
    """

    def dump(self, obj, file):
        file = ensure_filetype(file, self.extension)
        with open(file, 'w+') as ofile:
            yaml.dump(obj, ofile)

    def load(self, file):
        file = ensure_filetype(file, self.extension)
        with open(file, 'r') as ofile:
            data = yaml.safe_load(ofile)
        return data

    @property
    def extension(self):
        return ".yaml"

    @property
    def importstring(self):
        return "import yaml"

    @property
    def callstring(self):
        return "yaml"

    @property
    def dumpstring(self) -> str:
        return "dump"

    @property
    def loadstring(self) -> str:
        return "safe_load"

    def dumpfunc(self) -> str:
        lines = ['\ndef dump(obj, file):',
                 f'\t{self.importstring}',
                 f'\tif isinstance(obj, (set, tuple)):',
                 f'\t\tobj = list(obj)',
                 f'\tif not file.endswith("{self.extension}"):',
                 f'\t\tfile = file + "{self.extension}"',
                 f'\twith open(file, "{self.write_mode}") as o:',
                 f'\t\t{self.callstring}.{self.dumpstring}(obj, o)']

        return '\n'.join(lines)



from remotemanager.connection.url import URL
from remotemanager.logging import LoggingMixin
from remotemanager.connection.computers.options import required, optional
from remotemanager.logging.utils import format_iterable
from remotemanager.utils import safe_divide


class Example_Computer(URL, LoggingMixin):
    """
    example class for connecting to a remote computer
    """

    def __init__(self, **kwargs):

        if kwargs.get('host', None) is not None:
            raise ValueError('cannot change host of dedicated URL')

        kwargs['host'] = 'remote.address.for.connection'

        super().__init__(**kwargs)

        self.submitter = 'submit_command'
        self.shebang = '#!/bin/bash'
        self.pragma = '#PBS'

        self._options = {'mpi': required,
                         'omp': required,
                         'nodes': required,
                         'jobname': 'remote-run',
                         'queue': required,
                         'walltime': required,
                         'outfile': optional,
                         'errfile': optional}

        self.modules = ['python3']
        self.extra = None

    @property
    def ssh(self):
        """
        Returns (str):
            modified ssh string for Summer avoiding perl error
        """
        return 'LANG=C ' + super().ssh

    @property
    def required(self):
        """
        Returns the required arguments
        """
        return [k for k, v in self._options.items() if v == required]

    @property
    def arguments(self):
        return list(self._options.keys())

    def parse_resources(self, **kwargs):
        """
        Set any user given arguments, using defaults where not available

        Args:
            **kwargs:
                arguments for script generation

        Returns:
            None
        """
        for argument in self.arguments:
            val = kwargs.get(argument, None)

            if val is None:
                val = self._options.get(argument)

            # only set if val has a value, or is a non-required placeholder
            if val != required:
                self.__setattr__(argument, val)
                self._logger.info(f'set arg {argument}: {val}')

    def resources_block(self, **kwargs):
        self.parse_resources(**kwargs)

        outfile = self.outfile or f'{self.jobname}-stdout'
        errfile = self.errfile or f'{self.jobname}-stderr'

        options = {'-N': self.jobname,
                   '-q': self.queue,
                   '-o': outfile,
                   '-e': errfile,
                   '-l': f'nodes={self.nodes}:'
                         f'ppn={self.mpi},'
                         f'walltime={self.walltime}'}

        return [f'{self.pragma} {k} {v}' for k, v in options.items()]

    def modules_block(self):
        return [''] + [f'module load {m}' for m in self.modules]

    def script(self,
               **kwargs) -> str:
        """
        Takes job arguments and produces a valid jobscript

        Returns:
            (str):
                script
        """
        script = [self.shebang]

        script += self.resources_block(**kwargs)
        script += self.modules_block()

        postscript = f"""
cd $PBS_O_WORKDIR
export OMP_NUM_THREADS={self.omp}
"""

        script.append(postscript)

        if self.extra is not None:
            script.append(self.extra)

        return '\n'.join(script)
